﻿
Imports System.Math

Module Module_Ponderations

    Private ThirdsOctaves() As Single = {10, 12.5, 16, 20, 25, 31.5, 40, 50, 63, 80, _
                                         100, 125, 160, 200, 250, 315, 400, 500, 630, 800, _
                                         1000, 1250, 1600, 2000, 2500, 3150, 4000, 5000, 6300, 8000, _
                                         10000, 12500, 16000, 20000, 25000, 0, 0, 0, 0, 0}

    Private PonderationA() As Single = {-70.4, -63.4, -56.7, -50.5, -44.7, -39.4, -34.6, -30.2, -26.2, -22.5, _
                                        -19.1, -16.1, -13.4, -10.9, -8.6, -6.6, -4.8, -3.2, -1.9, -0.8, _
                                          0.0, 0.6, 1.0, 1.2, 1.3, 1.2, 1.0, 0.5, -0.1, -1.1, _
                                         -2.5, -4.3, -6.6, -9.3, -12.0, 0.0, 0.0, 0.0, 0.0, 0.0}

    Private PonderationB() As Single = {-38.2, -33.2, -28.5, -24.2, -20.4, -17.1, -14.2, -11.6, -9.3, -7.4, _
                                        -5.6, -4.2, -3.0, -2.0, -1.3, -0.8, -0.5, -0.3, -0.1, 0.0, _
                                          0.0, 0.0, 0.0, -0.1, -0.2, -0.4, -0.7, -1.2, -1.9, -2.9, _
                                         -4.3, -6.1, -8.4, -11.1, -14.0, 0.0, 0.0, 0.0, 0.0, 0.0}

    Private PonderationC() As Single = {-14.3, -11.2, -8.5, -6.2, -4.4, -3.0, -2.0, -1.3, -0.8, -0.5, _
                                         -0.3, -0.2, -0.1, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, _
                                          0.0, 0.0, -0.1, -0.2, -0.3, -0.5, -0.8, -1.3, -2.0, -3.0, _
                                         -4.4, -6.2, -8.5, -11.2, -14.0, 0.0, 0.0, 0.0, 0.0, 0.0}

    Private PonderationD() As Single = {-26.5, -24.5, -22.5, -20.5, -18.5, -16.5, -14.5, -12.5, -11.0, -9.0, _
                                         -7.5, -6.0, -4.5, -3.0, -2.0, -1.0, -0.5, 0.0, 0.0, 0.0, _
                                          0.0, 2.0, 5.5, 8.0, 10.0, 11.0, 11.0, 10.0, 8.5, 6.0, _
                                          3.0, 0.0, -4.0, -7.5, -12.0, 0.0, 0.0, 0.0, 0.0, 0.0}


    Friend Array_CentralBandFreq() As Single
    Private Array_LowBandFreq() As Single
    Friend Array_HiBandFreq(2000) As Single
    Friend LastBandHiFrequency As Single

    ' ==========================================================================================
    '  USED BY METER
    ' ==========================================================================================
    Friend LevelA As Double
    Friend LevelB As Double
    Friend LevelC As Double
    Friend LevelD As Double
    Friend LevelZ As Double

    Friend Sub CalcPonderations(ByVal BandValues() As Single, _
                                ByVal mindb As Single, _
                                ByVal maxdb As Single)
        ' ----------------------------------------------------------- correct for MinDB MaxDB
        Dim bv(NumBands) As Double
        For i As Int32 = 0 To NumBands - 1
            bv(i) = mindb + BandValues(i) * (maxdb - mindb)
        Next
        ' ----------------------------------------------------------- calc ponderations
        LevelA = Calc_RowSoundLevel(AddPonderation(bv, PonderationA))
        LevelB = Calc_RowSoundLevel(AddPonderation(bv, PonderationB))
        LevelC = Calc_RowSoundLevel(AddPonderation(bv, PonderationC))
        LevelD = Calc_RowSoundLevel(AddPonderation(bv, PonderationD))
        LevelZ = Calc_RowSoundLevel(bv)

        'SmoothValue_Pow_Adaptive(LevelA, Calc_RowSoundLevel(AddPonderation(bv, PonderationA)), 5)
        'SmoothValue_Pow_Adaptive(LevelB, Calc_RowSoundLevel(AddPonderation(bv, PonderationB)), 5)
        'SmoothValue_Pow_Adaptive(LevelC, Calc_RowSoundLevel(AddPonderation(bv, PonderationC)), 5)
        'SmoothValue_Pow_Adaptive(LevelD, Calc_RowSoundLevel(AddPonderation(bv, PonderationD)), 5)
        'SmoothValue_Pow_Adaptive(LevelZ, Calc_RowSoundLevel(bv), 5)
    End Sub

    ' ==========================================================================================
    '  USED BY ELABORATION
    ' ==========================================================================================
    Friend Function AddPondA(ByVal InArray() As Double) As Double()
        Return AddPonderation(InArray, PonderationA)
    End Function

    Friend Function AddPondC(ByVal InArray() As Double) As Double()
        Return AddPonderation(InArray, PonderationC)
    End Function

    ' ==========================================================================================
    '  BANDWITH-DESIGNATOR and BAND-DENOMINATOR
    ' ==========================================================================================
    ' ------------------------------------------------------------------------------------------
    'The range of frequencies at which weighting can be carried out (also by interpolation) is 10-25000 Hz. 
    'The frequency bands for octaves and fractions of an octave are always calculated with  IEC 61260 
    '(OCTAVE-BAND AND FRACTIONAL-OCTAVE-BAND FILTERS) within the lower limit of the lowest band (22.4 Hz) 
    'and the upper limit of the highest band (22400 Hz) so there is no need to make arbitrary extrapolations 
    'outside the weighting range.
    ' ------------------------------------------------------------------------------------------
    ' 10 bands from 31.6 to 15848.9 Hz. 11 bands includes one lower frequency (central band), 15.8 Hz
    ' 30 bands from 25.1 to 19952.6. 31 bands includes one lower frequency (central band), 20.0 Hz. 
    ' 32 bands includes one other lower frequency (central band), 15.8 Hz, and so on.
    ' 60 bands from 23.7 to 21134.9. 61 bands includes one lower frequency (central band), 21.1 Hz. 
    ' 62 bands includes one other lower frequency (central band), 18.8 Hz, and so on.
    ' 120 bands from 23.0 to 21752.0 Hz
    ' ------------------------------------------------------------------------------------------
    Private Function BandDenominator() As Single
        Select Case NumBands
            Case 10, 11 : Return 1
            Case 20, 21, 22 : Return 2
            Case 30, 31, 32, 33, 34 : Return 3
            Case 40, 41, 42, 43, 44 : Return 4
            Case 50, 51, 52, 53, 54, 55 : Return 5
            Case 60, 61, 62, 63, 64, 65, 66, 67 : Return 6
            Case 70, 71, 72, 73, 74, 75, 76, 77, 78 : Return 7
            Case 80, 81, 82, 83, 84, 85, 86, 87, 88, 89 : Return 8
            Case 90, 91, 92, 93, 94, 95, 96, 97, 98, 99 : Return 9
            Case 100, 101, 102, 103, 104, 105, 106, 107, 108, 109 : Return 10
            Case 110, 111, 112, 113, 114, 115, 116, 117, 118, 119 : Return 11
            Case 120 : Return 12
            Case Else : Return CSng(NumBands / 10)
        End Select
    End Function

    Friend Function StandardizedBandNumber() As Single
        Return BandDenominator() * 10
    End Function

    Private Sub InfoBands()
        Dim mb As Single = NumBands - StandardizedBandNumber()
        Dim MoreBands As String = ""
        If mb > 0 Then
            MoreBands = " + " + mb.ToString + " Low B."
        End If
        Form1.Lbl_OctaveFraction.Text = "1/" + BandDenominator.ToString("0.#", GCI) + " Oct." + MoreBands
    End Sub

    ' ===================================================================================================
    '  ADD PONDERATION
    ' ===================================================================================================
    Private Function AddPonderation(ByVal InArray() As Double, ByVal PondArray() As Single) As Double()
        InfoBands()
        ' --------------------------------------------------------- Calculate Central Band Frequency Array 
        ReDim Array_CentralBandFreq(2000)
        ReDim Array_LowBandFreq(2000)
        Dim k_Odd As Single = CSng((10 ^ (3 / (StandardizedBandNumber()))))
        '
        ' The "Bandwidth Designator" is the fraction: 1 / BandDenominator
        '  1/1 designate bands spaced as octaves
        '  1/3 designate bands spaced as 1/3 of octave
        '  1/10 designate bands spaced as 1/10 of octave and so on
        '
        ' When the denominator of the bandwidth designator is an odd number, use the first formula, else the second
        '
        If BandDenominator() Mod 2 = 1 Then
            For i As Int32 = 0 To NumBands
                Array_CentralBandFreq(1000 + i) = CSng(1000 * (k_Odd ^ i))
                Array_CentralBandFreq(1000 - i) = CSng(1000 * (k_Odd ^ -i))
            Next
        Else
            For i As Int32 = 0 To NumBands
                Array_CentralBandFreq(1000 + i) = CSng(1000 * (k_Odd ^ i) / (k_Odd ^ 0.5))
                Array_CentralBandFreq(1000 - i) = CSng(1000 * (k_Odd ^ -i) / (k_Odd ^ 0.5))
            Next
        End If
        ' --------------------------------------------------------- Sort Array of Central Band Frequency
        Dim max As Int32 = 22400
        Dim imax As Int32
        For i As Int32 = 0 To Array_CentralBandFreq.Length - 1
            If Array_CentralBandFreq(i) * k_Odd ^ (0.5) > max Then
                imax = i - 1
                Exit For
            End If
        Next
        Dim imin As Int32 = imax - NumBands
        For i As Int32 = imin To imax - 1
            Array_CentralBandFreq(i - imin) = Array_CentralBandFreq(i + 1)
            Array_LowBandFreq(i - imin) = CSng(Array_CentralBandFreq(i - imin) * k_Odd ^ (-0.5))
            Array_HiBandFreq(i - imin) = CSng(Array_CentralBandFreq(i - imin) * k_Odd ^ (0.5))
        Next
        Array_LowBandFreq(imax - imin) = CSng(Array_CentralBandFreq(imax + 1) * 2 ^ (-1 / (2 * NumBands / 10))) 'last (dx) external High frequency
        Dim Frequency_External_Interval As Single = CSng(Array_CentralBandFreq(NumBands - 1) * k_Odd ^ (0.5))
        ReDim Preserve Array_CentralBandFreq(NumBands - 1)

        LastBandHiFrequency = CSng(Array_CentralBandFreq(NumBands - 1) * k_Odd ^ (0.5))

        ' -------------------------------------------------------------------- Interpolated Ponderation
        Dim ar(NumBands - 1) As Double
        For i As Int32 = 0 To NumBands - 1
            ar(i) = InArray(i) + Interpolate(Array_CentralBandFreq(i), _
                                             ThirdsOctaves, _
                                             PondArray)
        Next
        '
        Return ar
    End Function

    Friend Function Interpolate(ByVal x_to_interpolate As Single, ByVal X() As Single, ByVal Y() As Single) As Single
        For i As Int32 = 0 To X.Length - 2
            If x_to_interpolate <= X(i + 1) And x_to_interpolate > X(i) Then
                Return ((Y(i + 1) - Y(i)) * (x_to_interpolate - X(i))) / (X(i + 1) - X(i)) + Y(i)
            End If
        Next
    End Function

    ' ===================================================================================================
    '  RowSoundLevel  - integrale per riga (frequenze)
    ' ===================================================================================================
    Private RowBandsCorrection As Double = 0

    Friend Function Calc_RowSoundLevel(ByVal L() As Double) As Double
        Dim sum As Double = 0
        ' ------------------------------------------------------------- POWER 10  (-10dB -10dB)
        'For i As Int32 = 0 To NumBands - 1
        '    sum += 10 ^ (0.1 * L(i))
        'Next
        'Return 10 * Log10(sum / NumBands)
        ' ------------------------------------------------------------- POWER 10 NODIV --- OK
        For i As Int32 = 0 To NumBands - 1
            sum += 10 ^ (0.1 * L(i))
        Next
        Return 10 * Log10(sum)
        ' ------------------------------------------------------------- POWER 10 and SQRT of NumBands (-4dB -4dB)
        'For i As Int32 = 0 To NumBands - 1
        '    sum += 10 ^ (0.1 * L(i))
        'Next
        'Return 10 * Log10(sum / Sqrt(NumBands))
        ' ------------------------------------------------------------- POWER 10 and SQUARED and SQRT  (-3dB -5dB)
        'For i As Int32 = 0 To NumBands - 1
        '    sum += (10 ^ (0.1 * L(i))) ^ 2
        'Next
        'Return 10 * Log10(Sqrt(sum))
        ' ------------------------------------------------------------- SQUARED and SQRT of MID (-12dB -12dB)
        'For i As Int32 = 0 To NumBands - 1
        '    sum += L(i) * L(i)
        'Next
        'Return Sqrt(sum / NumBands)
    End Function

    'Leq sound levels are logarithmic (dB) values and cannot be added directly. 
    'A doubling of sound level results in a measured increase of 3 dB, 
    'four identical sources in a room would increase the noise level by 6 dB and so on. 
    'This works both ways, say 10 similar machines in a room produce 100 dBA 
    'then removing one machine completely will only reduce the overall noise level to 0.5 dBA, 
    'you would need to silence or remove 50% of the machines to achieve a 3 dB reduction.

End Module
