﻿
Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False

    Friend GCI As Globalization.CultureInfo = Globalization.CultureInfo.InvariantCulture

    ' =======================================================================================================
    '   APP TITLE AND VERSION (ALL FIELDS)
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal RemoveThereminoPrefix As Boolean = False) As String
        Dim appname As String = IO.Path.GetFileNameWithoutExtension(Application.ExecutablePath)
        Dim Title As String = Replace(appname, "_", " ")
        If RemoveThereminoPrefix Then Title = Replace(Title, "Theremino ", "")
        Return Title + " - V" + GetVersionString()
    End Function

    ' =======================================================================================================
    '   GET VERSION (ALSO TRIPLE OR QUADRUPLE FIELDS)
    ' =======================================================================================================
    Friend Function GetVersionString() As String
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        GetVersionString = s(0) + "." + s(1)
        If s(2) <> "0" Then
            GetVersionString += "." + s(2)
            If s(3) <> "0" Then
                GetVersionString += "." + s(3)
            End If
        End If
    End Function

    ' =======================================================================================
    '  FORM FUNCTIONS
    ' =======================================================================================
    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
        'EnsureFormCompletelyVisible(f)
    End Sub

    Private SB As Rectangle = New Rectangle(Integer.MaxValue, Integer.MaxValue, Integer.MinValue, Integer.MinValue)

    Private Sub GetMaxScreenBounds()
        For Each s As Screen In System.Windows.Forms.Screen.AllScreens
            SB = Rectangle.Union(SB, s.WorkingArea)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, SB.Width)         ' not more than a maximized window
    '        .Height = Math.Min(.Height, SB.Height)      ' not more than a maximized window
    '        .Width = Math.Max(.Width, 32)               ' at least 32x24
    '        .Height = Math.Max(.Height, 24)             ' at least 32x24
    '        .Left = Math.Min(.Left, SB.Right - .Width)  ' not beyond the right border
    '        .Top = Math.Min(.Top, SB.Bottom - .Height)  ' not beyond the bottom border
    '        .Left = Math.Max(.Left, SB.Left)            ' at least at the left border
    '        .Top = Math.Max(.Top, SB.Top)               ' at least at the top border
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)             ' not more than VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, SB.Height)          ' not more than VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)                   ' at least 32x24
            .Height = Math.Max(.Height, 24)                 ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - 50)          ' not beyond right border - 50 pixels
            .Top = Math.Min(.Top, SB.Bottom - 100)          ' not beyond bottom border - 50 pixels
            .Left = Math.Max(.Left, SB.Left + 150 - .Width) ' at least at left border + 150 pixels
            .Top = Math.Max(.Top, SB.Top - 10)              ' at least at top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================
    Private Function TabString(ByVal Name As String, _
                               Optional ByVal Value As Double = Double.NaN, _
                               Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt, GCI)
        End If
    End Function
    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function
    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function
    Private Function Val_Double(ByVal l As String) As Double
        Return Val(l.Replace(",", "."))
    End Function
    Friend Function Val_Single(ByVal l As String) As Single
        Return CSng(Val(l.Replace(",", ".")))
    End Function
    Friend Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function
    Private Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Int32
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function
    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function


    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================
    Friend Sub SaveConfigurationAs()
        InitConfigFilesFolder()
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.InitialDirectory = ConfigFilesFolder
        sfd.DefaultExt = ".txt"
        sfd.Filter = "Configuration file (*.txt)|*txt"
        'sfd.FileName = AssemblyName() & "_INI_" & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        sfd.FileName = "Configuration_" & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Save_INI(sfd.FileName)
        End If
    End Sub

    Friend Sub LoadConfiguration()
        InitConfigFilesFolder()
        Dim ofd As OpenFileDialog = New OpenFileDialog
        ofd.InitialDirectory = ConfigFilesFolder
        ofd.Filter = "Configuration file (*.txt)|*txt"
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Load_INI(ofd.FileName)
        End If
    End Sub

    Friend Sub Save_INI(Optional ByVal iniFileName As String = "")
        '
        If iniFileName = "" Then
            iniFileName = Application.StartupPath & "\" + AssemblyName() + "_INI.txt"
        End If
        '
        Dim f As System.IO.StreamWriter = Nothing
        Try
            f = IO.File.CreateText(iniFileName)
            '
            f.WriteLine(" Window position and size")
            f.WriteLine("===========================================")
            Dim r As Rectangle
            r = GetFormRectangle(Form1)
            f.WriteLine(TabString("Form1_Top", r.Top))
            f.WriteLine(TabString("Form1_Left", r.Left))
            f.WriteLine(TabString("Form1_Width", r.Width))
            f.WriteLine(TabString("Form1_Height", r.Height))
            f.WriteLine(TabString("Form1_WindowState", Form1.WindowState))
            f.WriteLine(TabString("SplitterDistance", Form1.SplitContainer1.SplitterDistance))
            '
            '
            f.WriteLine("")
            f.WriteLine(" Controls")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("FilePathAndName", Form1.FilePathAndName))
            f.WriteLine(TabString("PeriodsCount", Form1.txt_PeriodsCount.Text))
            f.WriteLine(TabString("DayStart", Form1.txt_DayStart.Text))
            f.WriteLine(TabString("NightStart", Form1.txt_NightStart.Text))
            '
            f.WriteLine(TabString("GroupBox1_Height", Form1.GroupBox1.Height))
            f.WriteLine(TabString("GroupBox2_Height", Form1.GroupBox2.Height))
            f.WriteLine(TabString("GroupBox3_Height", Form1.GroupBox3.Height))
            f.WriteLine(TabString("GroupBox4_Height", Form1.GroupBox4.Height))
            f.WriteLine(TabString("GroupBox5_Height", Form1.GroupBox5.Height))
            f.WriteLine(TabString("GroupBox6_Height", Form1.GroupBox6.Height))
            '
            f.WriteLine(TabString("ReportsInterval", Form1.txt_ReportsInterval.Text))
            f.WriteLine(TabString("LAeqInterval", Form1.txt_LAeqInterval.Text))
            f.WriteLine(TabString("PercentileInterval", Form1.txt_PercentileInterval.Text))
            f.WriteLine(TabString("SpecMaxDb", Form1.txt_SpecMaxDb.Text))
            f.WriteLine(TabString("SpecMinDb", Form1.txt_SpecMinDb.Text))
            f.WriteLine(TabString("LoudnessType", LoudnessType))
            f.WriteLine(TabString("TC_MaxTimeWithout", Form1.txt_TC_MaxTimeWithout.Text))
            f.WriteLine(TabString("TC_MinTimeFor", Form1.txt_TC_MinTimeFor.Text))
            f.WriteLine(TabString("SubtractOutliersCoeff", Form1.txt_SubtractOutliersCoeff.Text))
            f.WriteLine(TabString("SelMaxSeconds", Form1.txt_SelMaxSeconds.Text))
            '
            f.WriteLine(TabString("GraphScale", GraphPaper1.ScaleHorizontal))
            f.WriteLine(TabString("DeltaSamples", DeltaSamples))
            f.WriteLine(TabString("ShowFrequencyGraph", Form1.chk_ShowFrequencyGraph.Checked))
            f.WriteLine(TabString("AutoMax", Form1.chk_AutoMax.Checked))
            f.WriteLine(TabString("LoudnessSelectedBand", LoudnessSelectedBand))
            '
            f.WriteLine("")
            f.WriteLine(" Filters Params")
            f.WriteLine("===========================================")
            f.WriteLine(TabString("FilterImpulseInc", Form1.txt_FilterImpulseInc.Text))
            f.WriteLine(TabString("FilterImpulseDec", Form1.txt_FilterImpulseDec.Text))
            f.WriteLine(TabString("FilterFastInc", Form1.txt_FilterFastInc.Text))
            f.WriteLine(TabString("FilterFastDec", Form1.txt_FilterFastDec.Text))
            f.WriteLine(TabString("FilterSlowInc", Form1.txt_FilterSlowInc.Text))
            f.WriteLine(TabString("FilterSlowDec", Form1.txt_FilterSlowDec.Text))
            f.WriteLine(TabString("FilterImpulseLinear", Form1.chk_FilterImpulseLinear.Checked))
            f.WriteLine(TabString("FilterFastLinear", Form1.chk_FilterFastLinear.Checked))
            f.WriteLine(TabString("FilterSlowLinear", Form1.chk_FilterSlowLinear.Checked))
            '
            f.WriteLine(TabString("FtpFolder", Form1.txt_FtpFolder.Text))
            f.WriteLine(TabString("FtpUserName", Form1.txt_FtpUserName.Text))
            f.WriteLine(TabString("FtpPassword", Form1.txt_FtpPassword.Text))
            f.WriteLine(TabString("SendToFtp", Form1.chk_SendToFtp.Checked))
            '
            f.WriteLine(TabString("SendMail_Destinations", Form1.txt_SendMail_Destinations.Text))
            f.WriteLine(TabString("SendMail_SMTP", Form1.txt_SendMail_SMTP.Text))
            f.WriteLine(TabString("SendMail_Sender", Form1.txt_SendMail_Sender.Text))
            f.WriteLine(TabString("SendMail_Password", Form1.txt_SendMail_Password.Text))
            f.WriteLine(TabString("SendMail_MaxLAeq", Form1.txt_SendMails_MaxLAeq.Text))
            f.WriteLine(TabString("SendMail_Encripted", Form1.chk_SendMail_Encripted.Checked))
            f.WriteLine(TabString("PercentilesLevels", Form1.Txt_Percentiles_levels.Text))
            '
            f.WriteLine(TabString("RoundDisplayFreq", Form1.chk_RoundDisplayFreq.Checked))
        Catch
        End Try
        Try
            f.Close()
        Catch
        End Try
    End Sub


    Friend Sub Load_INI(Optional ByVal iniFileName As String = "")
        '
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' 
        Dim LoadAll As Boolean = False
        If iniFileName = "" Then
            iniFileName = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
            LoadAll = True
        End If
        '
        ' -------------------------------------------------------------------------------
        Dim l As String
        Dim pname As String
        If My.Computer.FileSystem.FileExists(iniFileName) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)
            Do While Not f.EndOfStream
                l = f.ReadLine()
                pname = ExtractParamName(l)
                If LoadAll Then
                    Select Case pname
                        Case "Form1_Top" : Form1.Top = Val_Int(l)
                        Case "Form1_Left" : Form1.Left = Val_Int(l)
                        Case "Form1_Width" : Form1.Width = Val_Int(l)
                        Case "Form1_Height" : Form1.Height = Val_Int(l)
                        Case "Form1_WindowState" : Form1.WindowState = CType((Val(l)), FormWindowState)
                        Case "SplitterDistance" : Form1.SplitContainer1.SplitterDistance = Val_Int(l)
                    End Select
                End If
                Select Case pname
                    Case "FilePathAndName" : Form1.FilePathAndName = l
                    Case "PeriodsCount" : Form1.txt_PeriodsCount.Text = l
                    Case "DayStart" : Form1.txt_DayStart.Text = l
                    Case "NightStart" : Form1.txt_NightStart.Text = l
                        '
                    Case "GroupBox1_Height" : Form1.GroupBox1.Height = Val_Int(l)
                    Case "GroupBox2_Height" : Form1.GroupBox2.Height = Val_Int(l)
                    Case "GroupBox3_Height" : Form1.GroupBox3.Height = Val_Int(l)
                    Case "GroupBox4_Height" : Form1.GroupBox4.Height = Val_Int(l)
                    Case "GroupBox5_Height" : Form1.GroupBox5.Height = Val_Int(l)
                    Case "GroupBox6_Height" : Form1.GroupBox6.Height = Val_Int(l)
                        '
                    Case "ReportsInterval" : Form1.txt_ReportsInterval.Text = l
                    Case "LAeqInterval" : Form1.txt_LAeqInterval.Text = l
                    Case "PercentileInterval" : Form1.txt_PercentileInterval.Text = l
                    Case "SpecMaxDb" : Form1.txt_SpecMaxDb.Text = l
                    Case "SpecMinDb" : Form1.txt_SpecMinDb.Text = l
                    Case "LoudnessType" : LoudnessType = CType(Val_Int(l), LoudnessTypes)
                    Case "TC_MaxTimeWithout" : Form1.txt_TC_MaxTimeWithout.Text = l
                    Case "TC_MinTimeFor" : Form1.txt_TC_MinTimeFor.Text = l
                    Case "SubtractOutliersCoeff" : Form1.txt_SubtractOutliersCoeff.Text = l
                    Case "SelMaxSeconds" : Form1.txt_SelMaxSeconds.Text = l
                        '
                    Case "GraphScale" : GraphPaper1.SetHorizontalScale(Val_Single(l))
                    Case "DeltaSamples" : DeltaSamples = Val_Single(l)
                    Case "ShowFrequencyGraph" : Form1.chk_ShowFrequencyGraph.Checked = l = "True"
                    Case "AutoMax" : Form1.chk_AutoMax.Checked = l = "True"
                    Case "LoudnessSelectedBand" : LoudnessSelectedBand = Val_Int(l)
                        '
                    Case "FilterImpulseInc" : Form1.txt_FilterImpulseInc.Text = l
                    Case "FilterImpulseDec" : Form1.txt_FilterImpulseDec.Text = l
                    Case "FilterFastInc" : Form1.txt_FilterFastInc.Text = l
                    Case "FilterFastDec" : Form1.txt_FilterFastDec.Text = l
                    Case "FilterSlowInc" : Form1.txt_FilterSlowInc.Text = l
                    Case "FilterSlowDec" : Form1.txt_FilterSlowDec.Text = l
                    Case "FilterImpulseLinear" : Form1.chk_FilterImpulseLinear.Checked = l = "True"
                    Case "FilterFastLinear" : Form1.chk_FilterFastLinear.Checked = l = "True"
                    Case "FilterSlowLinear" : Form1.chk_FilterSlowLinear.Checked = l = "True"
                        '
                    Case "FtpFolder" : Form1.txt_FtpFolder.Text = l
                    Case "FtpUserName" : Form1.txt_FtpUserName.Text = l
                    Case "FtpPassword" : Form1.txt_FtpPassword.Text = l
                    Case "SendToFtp" : Form1.chk_SendToFtp.Checked = CBool(l)
                        '
                    Case "SendMail_Destinations" : Form1.txt_SendMail_Destinations.Text = l
                    Case "SendMail_SMTP" : Form1.txt_SendMail_SMTP.Text = l
                    Case "SendMail_Sender" : Form1.txt_SendMail_Sender.Text = l
                    Case "SendMail_Password" : Form1.txt_SendMail_Password.Text = l
                    Case "SendMail_MaxLAeq" : Form1.txt_SendMails_MaxLAeq.Text = l
                    Case "SendMail_Encripted" : Form1.chk_SendMail_Encripted.Checked = CBool(l)
                    Case "PercentilesLevels" : Form1.Txt_Percentiles_levels.Text = l
                        '
                    Case "RoundDisplayFreq" : Form1.chk_RoundDisplayFreq.Checked = CBool(l)
                End Select
            Loop
            f.Close()
        End If
        LimitFormPosition(Form1)
    End Sub

    Friend ConfigFilesFolder As String
    Friend Sub InitConfigFilesFolder()
        ConfigFilesFolder = Application.StartupPath & "\ConfigFiles"
        IO.Directory.CreateDirectory(ConfigFilesFolder)
    End Sub


End Module
