﻿
Namespace AudioMeter

    Module Module_AudioMeter

        Private m_FourierSamples As Int32 = 8192
        Private m_SamplesPerSec As Int32 = 50000
        Private m_Channels As Int32 = 1
        Private m_NumBuffers As Int32 = 16
        Private m_LenBuffers As Int32 = 1000

        Private m_fifo As FIFO = New FIFO(200000)
        Private m_fht As FHT = New FHT
        Private m_Recorder As AudioIn
        Private m_InputBuffer(-1) As Int16
        Private m_StreamWriter As IO.StreamWriter
        Private m_FileStream As IO.FileStream

        Friend WindowType As FHT.WindowTypes = FHT.WindowTypes.FlatTop

        ' ================================================================================================
        '  FAST INIT
        ' ================================================================================================
        Friend Sub Initialize(ByVal InputDeviceIndex As Int32, _
                              ByVal nsamples As Int32)
            '
            m_FourierSamples = nsamples
            '
            ' --------------------------------------------------------- init and create OutFolder
            InitAndCreateFolders()
            '
            ' --------------------------------------------------------- buffer size for sampling milliseconds
            If MeterSamplesMillisec < 10 Then MeterSamplesMillisec = 10
            If MeterSamplesMillisec > 100 Then MeterSamplesMillisec = 100
            m_LenBuffers = MeterSamplesMillisec * 50
            ' --------------------------------------------------------- 
            CloseAll()
            ReDim m_InputBuffer(m_FourierSamples - 1)
            m_Recorder = New AudioIn()
            m_Recorder.InputOn(InputDeviceIndex, _
                               m_SamplesPerSec, _
                               m_Channels, _
                               m_LenBuffers, _
                               m_NumBuffers, _
                               New AudioIn.BufferDoneEventHandler(AddressOf DataArrived))
            ' --------------------------------------------------------- 
            Threading.Thread.Sleep(100 + MeterSamplesMillisec)
            OutFileEnabled = True
        End Sub

        ' ===================================================================================
        '  This function creates the "Theremino_Files" folder outside the application folder
        '  The folder is created one or two levels UP, starting from the exe folder
        '  depending if the app has the sources or not
        ' -----------------------------------------------------------------------------------
        '  The final folder structure is:
        '    Theremino_Phonometer
        '       Theremino_AudioElaboration
        '       Theremino_AudioMeter
        '       Theremino_Files
        ' -----------------------------------------------------------------------------------
        Private DataFilePath As String
        Friend ReportsPath As String
        '
        Private Sub InitAndCreateFolders()
            Dim path As String = Application.StartupPath
            path = IO.Directory.GetParent(path).ToString
            If path.Contains(My.Application.Info.AssemblyName) Then
                path = IO.Directory.GetParent(path).ToString
            End If
            DataFilePath = path + "\Theremino_Files\RawData"
            IO.Directory.CreateDirectory(DataFilePath)
            ReportsPath = path + "\Theremino_Files\Reports"
            IO.Directory.CreateDirectory(ReportsPath)
        End Sub
        ' ===================================================================================

        Friend Sub CloseAll()
            If m_Recorder IsNot Nothing Then
                m_Recorder.InputOff()
                m_Recorder = Nothing
                CloseOutputFile()
            End If
        End Sub

        Private LockObj As Object = New Object
        Private Sub DataArrived(ByRef data() As Int16)
            SyncLock LockObj
                ' ----------------------------------------------- write immediately to get the best time precision
                WriteSlotsAndFile(NumBands, m_fht.BandsBuffer)
                ' ----------------------------------------------- 
                m_fifo.AddArray(data)
                RunMeter()
            End SyncLock
        End Sub

        ' ================================================================================================
        '  METER EXECUTION
        ' ================================================================================================
        Private Sub RunMeter()
            m_fifo.SetReadIndex(m_fifo.GetWriteIndex - m_InputBuffer.Length)
            m_fifo.GetArray(m_InputBuffer)
            m_fht.ExecuteFHT(m_InputBuffer, WindowType)
            SetBandEqualizationsFromCalibrator()
            m_fht.ExtractBands(NumBands, Calibration, MinFreq, MaxFreq, MindB, MaxdB, LogX, m_SamplesPerSec, 1)
        End Sub

        ' ----------------------------------------------------------- millisec from NOW
        'Private s As String
        'Private Sub WriteSlotsAndFile(ByVal numbands As Int32, ByVal bandsBuffer() As Single)
        '    Try
        '        s = Now.ToString("HH:mm:ss fff")
        '        For i As Int32 = 0 To numbands - 1
        '            Dim dB As Single = MindB + bandsBuffer(i) * (MaxdB - MindB)
        '            If dB < 0 Then dB = 0
        '            If FirstSlot > 0 Then Slots.WriteSlot(FirstSlot + i, dB)
        '            s += vbTab + dB.ToString("00.0", GCI)
        '        Next
        '        If WriteToFile Then
        '            TestFile()
        '            m_StreamWriter.Write(s + vbCrLf)
        '        End If
        '    Catch
        '    End Try
        'End Sub

        ' ---------------------------------------------------------- sync millisec with audio buffers
        Private str As String
        Private millisec As Int32
        Private DateNow As DateTime
        Private oldSecond As Int32
        Private Sub WriteSlotsAndFile(ByVal numbands As Int32, ByVal bandsBuffer() As Single)
            Try
                DateNow = Now
                If DateNow.Second <> oldSecond Then
                    oldSecond = DateNow.Second
                    millisec = 0
                End If
                str = DateNow.ToString("HH:mm:ss ") + millisec.ToString("000")
                For i As Int32 = 0 To numbands - 1
                    Dim dB As Single = MindB + bandsBuffer(i) * (MaxdB - MindB)
                    If dB < 0 Then dB = 0
                    If FirstSlot > 0 Then Slots.WriteSlot(FirstSlot + i, dB)
                    str += vbTab + dB.ToString("00.0", GCI)
                Next
                If WriteToFile Then
                    TestFile()
                    m_StreamWriter.Write(str + vbCrLf)
                End If
                millisec += MeterSamplesMillisec
                If millisec >= 1000 Then millisec = 0
            Catch
            End Try
        End Sub


        ' ================================================================================================
        '  OPUTPUT FILES
        ' ================================================================================================
        Private OutFileName As String = ""
        Private OutFileDate As String
        Private OutFileEnabled As Boolean = False

        Private Sub TestFile()
            Dim s As String = Now.ToString("yyyy-MM-dd")
            If s <> OutFileDate Then
                If OutFileEnabled Then
                    CloseOutputFile()
                    OpenOutputFile()
                End If
            End If
        End Sub

        Private Sub CloseOutputFile()
            If m_StreamWriter IsNot Nothing Then
                m_StreamWriter.Close()
                m_StreamWriter = Nothing
            End If
            If m_FileStream IsNot Nothing Then
                m_FileStream.Close()
                m_FileStream = Nothing
            End If
            OutFileDate = ""
        End Sub

        Private Sub OpenOutputFile()
            SyncLock LockObj

                OutFileDate = Now.ToString("yyyy-MM-dd")

                OutFileName = OutFileDate + "_" + _
                              NumBands.ToString + "_" + _
                              MinFreq.ToString(GCI) + "_" + _
                              MaxFreq.ToString(GCI) + "_" + _
                              If(LogX, "Log_", "Lin_") + _
                              MeterSamplesMillisec.ToString + "mS_" + _
                              TestSite

                OutFileName = DataFilePath + "\" + OutFileName + ".txt"

                m_FileStream = IO.File.Open(OutFileName, _
                                            IO.FileMode.Append, _
                                            IO.FileAccess.Write, _
                                            IO.FileShare.ReadWrite Or IO.FileShare.Delete)

                m_StreamWriter = New IO.StreamWriter(m_FileStream, _
                                                     System.Text.Encoding.ASCII)

                m_StreamWriter.AutoFlush = True

            End SyncLock
        End Sub

        Friend Sub DeleteOutputFile()
            If OutFileName = "" Then Return
            AudioMeter.CloseOutputFile()
            IO.File.Delete(OutFileName)
        End Sub

        Friend Sub CloseAndReopenFile()
            If OutFileName = "" Then Return
            AudioMeter.CloseOutputFile()
            AudioMeter.OpenOutputFile()
        End Sub

        Friend Sub OpenOutFolder()
            Process.Start(DataFilePath)
        End Sub

        Private Sub SetBandEqualizationsFromCalibrator()
            If m_fht.BandsEqualizations Is Nothing Then Return
            If m_fht.BandsEqualizations.Length = NumBands Then
                For i As Int32 = 0 To NumBands - 1
                    m_fht.BandsEqualizations(i) = GetMicCalibrationValue(NumBands, i)
                Next
            End If
        End Sub

    End Module

End Namespace