﻿Module Calibrator


    ' =====================================================
    '  CALIBRATION POINTS
    ' =====================================================

    ' FILE EXAMPLE 
    '--------------------------------------------
    ' FREQ    GEN    MIC    NOTES
    '--------------------------------------------
    '31       0      0      10 BANDS
    '63       0      0      10 BANDS
    '125      0      0      10 BANDS
    '250      0      0      10 BANDS
    '500      0      0      10 BANDS
    '1000     0      0      10 BANDS
    '2000     0      0      10 BANDS
    '4000     0      0      10 BANDS
    '8000     0      0      10 BANDS
    '16000    0      0      10 BANDS
    '18000    0      0      test1

    Private Structure CalibrationPoint
        Dim Freq As Single
        Dim CalGen As Single
        Dim CalMic As Single
        Dim Notes As String
    End Structure

    Private CalibrationPoints(-1) As CalibrationPoint

    Private CalibrationsFileName As String = Application.StartupPath + "\Theremino_AudioMeter_Calibrations.txt"

    Friend Sub ReadCalibrationsFile()
        Dim index As Int32 = 0
        Dim l As String
        Dim sa As String()
        If IO.File.Exists(CalibrationsFileName) Then
            Try
                ReDim CalibrationPoints(-1)
                Dim f As Int32 = FreeFile()
                FileOpen(f, CalibrationsFileName, OpenMode.Input)
                Do While Not EOF(f)
                    l = LineInput(f)
                    l = ReplaceMultipleSpacesAndTabAndTrim(l)
                    sa = l.Split(" "c)
                    If sa.Length >= 3 Then
                        If Val(sa(0)) > 0 Then
                            index = CalibrationPoints.Length
                            ReDim Preserve CalibrationPoints(index)
                            With CalibrationPoints(index)
                                .Freq = CSng(Val(sa(0)))
                                .CalGen = CSng(Val(sa(1)))
                                .CalMic = CSng(Val(sa(2)))
                                .Notes = GetRemainingFields(sa)
                            End With
                        End If
                    End If
                Loop
                FileClose(f)
                FillOrigArrays()
            Catch
                MessageBox.Show("Error reading the calibrations file", _
                                "Message from AudioMeter", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End Try
        Else
            MessageBox.Show("Calibrations file not found", _
                            "Message from AudioMeter", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End If
    End Sub

    Private Function GetRemainingFields(ByVal sa As String()) As String
        Dim s As String = ""
        For i As Int32 = 3 To Sa.Length - 1
            s += sa(i) + " "
        Next
        Return s.Trim
    End Function

    Friend Sub WriteCalibrationsFile()
        IO.File.Delete(CalibrationsFileName)
        Dim f As Int32 = FreeFile()
        FileOpen(f, CalibrationsFileName, OpenMode.Output, OpenAccess.Write)
        PrintLine(f, "--------------------------------------------")
        PrintLine(f, " FREQ    GEN    MIC    NOTES")
        PrintLine(f, "--------------------------------------------")
        For i As Int32 = 0 To CalibrationPoints.Length - 1
            With CalibrationPoints(i)
                PrintLine(f, .Freq.ToString("0.#", GCI).PadRight(9) + _
                             .CalGen.ToString("0.#", GCI).PadRight(7) + _
                             .CalMic.ToString("0.#", GCI).PadRight(7) + _
                             .Notes)
            End With
        Next
        FileClose(f)
    End Sub

    Friend Sub EditCalibrationsFile()
        Form1.Opacity = 0.7
        CreateFile_IfNotExists_AndWithoutLockingIt(CalibrationsFileName)
        ProcessStartAndWait(CalibrationsFileName)
        Form1.Opacity = 1
        ReadCalibrationsFile()
    End Sub


    ' =================================================================================================
    '   CALIBRATION ARRAYS - INTERPOLATIONS
    ' =================================================================================================
    Friend Sub FillOrigArrays()
        For i As Int32 = 0 To CalibrationPoints.Length - 1
            With CalibrationPoints(i)
                CalibFreqOrig(i) = .Freq
                CalibGenOrig(i) = .CalGen
                CalibMicOrig(i) = .CalMic
            End With
        Next
    End Sub


    ' =================================================================================================
    '   INTERPOLATIONS
    ' =================================================================================================

    Private CalibFreqOrig(120) As Single
    Private CalibGenOrig(120) As Single
    Private CalibMicOrig(120) As Single

    Private CalibGenInterpolated(120) As Single
    Private CalibMicInterpolated(120) As Single

    Friend OldArrayFreq(2000) As Single
    Friend NewArrayFreq(2000) As Single

    Friend Array_LowBandFreq(2000) As Single
    Friend Array_HiBandFreq(2000) As Single
    Friend LastBandHiFrequency As Single

    Friend Sub InterpolateCalibrations(ByVal NewNumBands As Int32)
        Band_CenterFrequency(CalibrationPoints.Length, OldArrayFreq)
        Band_CenterFrequency(NewNumBands, NewArrayFreq)
        Dim fx, mic1, mic2, gen1, gen2 As Single
        For i As Int32 = 0 To NewNumBands - 1
            For j As Int32 = 0 To CalibrationPoints.Length - 2
                fx = NewArrayFreq(i)
                mic1 = CalibMicOrig(j)
                mic2 = CalibMicOrig(j + 1)
                'CalibMicInterpolated(i) = InterpolateFreq(fx, OldArrayFreq, CalibMicOrig)
                CalibMicInterpolated(i) = InterpolateFreq(fx, CalibFreqOrig, CalibMicOrig)
                gen1 = CalibGenOrig(j)
                gen2 = CalibGenOrig(j + 1)
                'CalibGenInterpolated(i) = InterpolateFreq(fx, OldArrayFreq, CalibGenOrig)
                CalibGenInterpolated(i) = InterpolateFreq(fx, CalibFreqOrig, CalibGenOrig)
            Next
        Next
    End Sub

    Friend Function InterpolateFreq(ByVal x_to_interpolate As Single, ByVal X() As Single, ByVal Y() As Single) As Single
        Dim Ncalib As Int32 = CalibrationPoints.Length
        '------------------------------------------------------------------------ Extrapolate
        If x_to_interpolate < X(0) Then
            Return ((Y(1) - Y(0)) * (x_to_interpolate - X(0))) / (X(1) - X(0)) + Y(0)
        End If
        '------------------------------------------------------------------------ Interpolate
        For i As Int32 = 0 To Ncalib - 2
            If x_to_interpolate <= X(i + 1) And x_to_interpolate > X(i) Then
                Return ((Y(i + 1) - Y(i)) * (x_to_interpolate - X(i))) / (X(i + 1) - X(i)) + Y(i)
            End If
        Next
        '------------------------------------------------------------------------ Extrapolate
        If x_to_interpolate > X(Ncalib) Then
            Return ((Y(Ncalib - 1) - Y(Ncalib - 2)) * (x_to_interpolate - X(Ncalib - 1))) / (X(Ncalib - 1) - X(Ncalib - 2)) + Y(Ncalib - 1)
        End If
        ''------------------------------------------------------------------------ Interpolate
        'For i As Int32 = 0 To Ncalib - 2
        '    If x_to_interpolate < X(i + 1) And x_to_interpolate >= X(i) Then
        '        Return ((Y(i + 1) - Y(i)) * (x_to_interpolate - X(i))) / (X(i + 1) - X(i)) + Y(i)
        '    End If
        'Next
        ''------------------------------------------------------------------------ Extrapolate
        'If x_to_interpolate >= X(Ncalib) Then
        '    Return ((Y(Ncalib - 1) - Y(Ncalib - 2)) * (x_to_interpolate - X(Ncalib - 1))) / (X(Ncalib - 1) - X(Ncalib - 2)) + Y(Ncalib - 1)
        'End If
    End Function


    ' ===================================================================================================
    '  CALCULATE CENTRAL FREQUENCY BANDS AS FUNCTION OF NUMBER OF BANDS
    ' ===================================================================================================
    Friend Sub Band_CenterFrequency(ByVal NumBands As Int32, ByRef Array() As Single)
        ' --------------------------------------------------------- Calculate Central Band Frequency Array 
        ' ---------- azzero tutto altrimenti riducendo in Nunbands col passaggio Byref viene tenuta memoria degli elementi ulteriori nell'array passatogli
        For i As Int32 = 0 To 120 '31/12/23
            Array(i) = 0 '31/12/23
            Array_LowBandFreq(i) = 0 '31/12/23
            Array_HiBandFreq(i) = 0 '31/12/23
        Next '31/12/23

        Dim k_Odd As Single = CSng((10 ^ (3 / (StandardizedBandNumber(NumBands)))))
        '
        ' The "Bandwidth Designator" is the fraction: 1 / BandDenominator
        '  1/1 designate bands spaced as octaves
        '  1/3 designate bands spaced as 1/3 of octave
        '  1/10 designate bands spaced as 1/10 of octave and so on
        '
        ' When the denominator of the bandwidth designator is an odd number, use the first formula, else the second
        '
        If BandDenominator(NumBands) Mod 2 = 1 Then
            For i As Int32 = 0 To NumBands
                Array(1000 + i) = CSng(1000 * (k_Odd ^ i))
                Array(1000 - i) = CSng(1000 * (k_Odd ^ -i))
            Next
        Else
            For i As Int32 = 0 To NumBands
                Array(1000 + i) = CSng(1000 * (k_Odd ^ i) / (k_Odd ^ 0.5))
                Array(1000 - i) = CSng(1000 * (k_Odd ^ -i) / (k_Odd ^ 0.5))
            Next
        End If
        ' --------------------------------------------------------- Sort Array of Central Band Frequency
        Dim max As Int32 = 22400
        Dim imax As Int32
        For i As Int32 = 0 To Array.Length - 1
            'If Array(i) > max Then '31/12/23
            If Array(i) * k_Odd ^ (0.5) > max Then '31/12/23 delimitiamo con estremo superiore dell'ultima banda non con il centro banda
                imax = i - 1
                Exit For
            End If
        Next
        Dim imin As Int32 = imax - NumBands
        For i As Int32 = imin To imax - 1
            Array(i - imin) = Array(i + 1)
            Array_LowBandFreq(i - imin) = CSng(Array(i - imin) * k_Odd ^ (-0.5))
            Array_HiBandFreq(i - imin) = CSng(Array(i - imin) * k_Odd ^ (0.5))
        Next
        LastBandHiFrequency = CSng(Array(NumBands - 1) * k_Odd ^ (0.5))
    End Sub


    ' =======================================================================================================
    '  Interpolation function
    ' -------------------------------------------------------------------------------------------------------
    '  Example (returned value x is 30)
    '   I1=1     ix=3     I2=5
    '   V1=10    x=??     V2=50
    ' =======================================================================================================
    Friend Function InterpolateAndExtrapolate(ByVal i1 As Single, ByVal i2 As Single, ByVal ix As Single, _
                                              ByVal v1 As Single, ByVal v2 As Single) As Single
        If i2 = i1 Then Return v1
        Return v1 + (v2 - v1) * (ix - i1) / (i2 - i1)
    End Function

    ' =================================================================================================
    '   CALIBRATION ARRAYS - READ WRITE VALUES
    ' =================================================================================================
    Friend Function GetGeneratorCalibrationValue(ByVal nbands As Int32, ByVal band As Int32) As Single
        Return CalibGenInterpolated(band)
    End Function
    Friend Function GetMicCalibrationValue(ByVal nbands As Int32, ByVal band As Int32) As Single
        Return CalibMicInterpolated(band)
    End Function

    Friend Sub SetGeneratorCalibrationValue(ByVal nbands As Int32, ByVal band As Int32, ByVal value As Single)
        SetCalibrationPoint(band, value, Single.MinValue)
    End Sub
    Friend Sub SetMicCalibrationValue(ByVal nbands As Int32, ByVal band As Int32, ByVal value As Single)
        SetCalibrationPoint(band, Single.MinValue, value)
    End Sub

    Private Sub SetCalibrationPoint(ByVal band As Int32, ByVal NewGenValue As Single, ByVal NewMicValue As Single)
        Dim b As Int32 = 0
        Dim i As Int32 = 0
        Do
            If CalibrationPoints(i).Notes.StartsWith("10") Then
                If b = band Then
                    If NewGenValue > Single.MinValue Then
                        CalibrationPoints(i).CalGen = NewGenValue
                    End If
                    If NewMicValue > Single.MinValue Then
                        CalibrationPoints(i).CalMic = NewMicValue
                    End If
                    WriteCalibrationsFile()
                    FillOrigArrays()
                    InterpolateCalibrations(10)
                    Exit Do
                Else
                    b += 1
                End If
            End If
            i += 1
        Loop Until i >= CalibrationPoints.Length
    End Sub

End Module
