﻿Imports System.Drawing.Drawing2D

Namespace SpectrumBands

    Module SpectrumBands

        ' ----------------------------------------------------------------------------------------
        '  If this class is primarly used for applications similar to psychedelic lights 
        '  it is better to increase the response with a short input buffer (and fourier samples)
        '  this is done at the expense of the low frequency resolution.
        '  A very short buffer is 2048 bytes and produces a delay of 46 mS
        '  A medium buffer is 4096 bytes and produces a delay of 92 mS
        '  A good bass frequency resolution 8192 buffer has a delay = 185 mS
        ' ----------------------------------------------------------------------------------------

        Private m_FourierSamples As Int32 = 8192 ' 8192 with 44100 or 16384 with 88200 or 32768 with 176400
        Private m_SamplesPerSec As Int32 = 50000 ' 44100 or 176400 or 192000 or 198450
        Private m_Channels As Int32 = 1
        Private m_NumBuffers As Int32 = 4
        Private m_LenBuffers As Int32 = 4096

        Private m_fifo As FIFO = New FIFO(200000)
        Friend m_fht As FHT = New FHT
        Private m_Recorder As AudioIn
        Private m_InputBuffer() As Int16

        Private m_ddx As Int32
        Private m_ddy As Int32
        Private m_windowtype As FHT.WindowTypes

        Private m_selectedpen As Pen = New Pen(Color.OrangeRed, 4)
        Private m_fillBrush As LinearGradientBrush
        Private m_TextBrush As Brush = Brushes.Azure

        Private Sub DataArrived(ByRef data() As Int16)
            m_fifo.AddArray(data)
        End Sub

        Private Function CreateFillBrush(ByVal pbox As PictureBox, _
                                         ByVal direction As LinearGradientMode) As LinearGradientBrush
            CreateFillBrush = New LinearGradientBrush(pbox.ClientRectangle, _
                                                      Color.Black, _
                                                      Color.Black, _
                                                      direction)
            Dim myBlend As ColorBlend = New ColorBlend()
            myBlend.Colors = New Color() {Color.FromArgb(255, 0, 0), _
                                          Color.FromArgb(255, 230, 0), _
                                          Color.FromArgb(250, 250, 0), _
                                          Color.FromArgb(230, 230, 0), _
                                          Color.FromArgb(0, 120, 0)}
            myBlend.Positions = New Single() {0.0F, 0.4F, 0.5F, 0.6F, 1.0F}
            CreateFillBrush.InterpolationColors = myBlend
        End Function

        Friend Sub Initialize(ByVal pbox As PictureBox, _
                              ByVal InputDeviceIndex As Int32, _
                              ByVal nsamples As Int32)
            '
            m_FourierSamples = nsamples
            '
            CloseAll()
            ReDim m_InputBuffer(m_FourierSamples - 1)
            m_fillBrush = CreateFillBrush(pbox, LinearGradientMode.Vertical)
            m_Recorder = New AudioIn()
            m_Recorder.InputOn(InputDeviceIndex, _
                               m_SamplesPerSec, _
                               m_Channels, _
                               m_LenBuffers, _
                               m_NumBuffers, _
                               New AudioIn.BufferDoneEventHandler(AddressOf DataArrived))
        End Sub

        Friend Sub CloseAll()
            If m_Recorder IsNot Nothing Then
                m_Recorder.InputOff()
                m_Recorder = Nothing
            End If
        End Sub

        Friend Sub Update(ByVal pbox As PictureBox, _
                          ByVal speed As Single, _
                          ByVal windowtype As FHT.WindowTypes)

            If m_Recorder Is Nothing Then Return
            '
            Static oldWriteIndex As Int32
            Dim newWriteIndex As Int32 = m_fifo.GetWriteIndex
            If newWriteIndex = oldWriteIndex Then Return
            oldWriteIndex = newWriteIndex
            '
            InitPictureboxImage(pbox)
            '
            If pbox.Image.Width <> m_ddx OrElse _
                                    pbox.Image.Height <> m_ddy OrElse _
                                    windowtype <> m_windowtype Then

                m_ddx = pbox.Image.Width
                m_ddy = pbox.Image.Height
                m_windowtype = windowtype
                speed = 1
            End If
            '
            Dim g As Graphics = Graphics.FromImage(pbox.Image)
            g.Clear(Color.FromArgb(60, 60, 60))
            '
            If MindB >= MaxdB Or MinFreq >= MaxFreq - 10 Or MaxFreq >= m_SamplesPerSec \ 2 Then
                g.DrawString("Invalid params", New Font("Arial", 11), m_TextBrush, 10, 10)
                pbox.Invalidate()
                Return
            End If
            '
            m_fifo.SetReadIndex(m_fifo.GetWriteIndex - m_InputBuffer.Length)
            m_fifo.GetArray(m_InputBuffer)
            m_fht.ExecuteFHT(m_InputBuffer, m_windowtype)
            SetBandEqualizationsFromCalibrator()
            m_fht.ExtractBands(NumBands, Calibration, MinFreq, MaxFreq, MindB, MaxdB, LogX, m_SamplesPerSec, speed)
            '
            Dim x As Single
            Dim h As Int32 = pbox.Image.Height
            Dim bandspacing As Single = CSng(pbox.Image.Width / NumBands)
            Dim bandsize As Single = bandspacing - 1
            If bandspacing < 4 Then bandsize = bandspacing
            ' --------------------------------------------------------------------------
            Dim freq As Single
            Dim klog As Single = CSng((MaxFreq / MinFreq) ^ (1 / (NumBands)))
            Dim klin As Single = CSng((MaxFreq - MinFreq) / (NumBands))
            If LogX Then
                freq = MinFreq * CSng(Math.Sqrt(klog))
            Else
                freq = MinFreq + klin / 2
            End If
            ' --------------------------------------------------------------------------
            Dim fs As Int32 = m_ddx \ 80
            If fs > 11 Then fs = 12
            If fs < 6 Then fs = 6
            Dim f As Font = New Font("Lucida console", fs)
            Dim f2 As Font = New Font("Lucida console", fs * 0.7F)
            Dim dx As Int32 = CInt(pbox.Image.Width / NumBands / 2) - fs \ 2 - 1
            Dim dy As Single
            Dim y As Single
            Dim strfreq As String
            Dim bandFreq As Single
            Dim db As Double
            For b As Int32 = 0 To NumBands - 1
                ' -------------------------------------------------------- draw bands
                x = b * bandspacing
                y = h * (1 - m_fht.BandsBuffer(b))
                If y < 0 Then y = 0
                g.FillRectangle(m_fillBrush, x + 1, y, bandsize, h)
                '
                ' -------------------------------------------------------- calibrations info
                If CalibPanelOpened Then
                    If b = CalibSelectedBand Then
                        g.DrawRectangle(m_selectedpen, x + 1, y, bandsize - 1, h)
                    End If
                    If NumBands > 10 Then
                        g.RotateTransform(-90)
                        db = GetGeneratorCalibrationValue(NumBands, b)
                        g.DrawString("Gen " + db.ToString("0.0", GCI), f2, Brushes.White, -y + 5, x + fs * 0.5F)
                        db = GetMicCalibrationValue(NumBands, b)
                        g.DrawString("Mic " + db.ToString("0.0", GCI), f2, Brushes.White, -y + 5, x + fs * 1.5F)
                        g.ResetTransform()
                    Else
                        db = GetGeneratorCalibrationValue(NumBands, b)
                        g.DrawString("Gen " + db.ToString("0.0", GCI), f, Brushes.White, x + fs, y - fs * 3.5F)
                        db = GetMicCalibrationValue(NumBands, b)
                        g.DrawString("Mic " + db.ToString("0.0", GCI), f, Brushes.White, x + fs, y - fs * 1.8F)
                    End If
                End If
                '
                ' -------------------------------------------------------- print frequencies
                If RoundDisplayFreq Then
                    bandFreq = RoundFreq(freq)
                    If freq >= 1000 Then
                        strfreq = bandFreq.ToString("0", GCI)
                    Else
                        strfreq = bandFreq.ToString("0.#", GCI)
                    End If
                Else
                    If freq >= 1000 Then
                        strfreq = freq.ToString("0", GCI)
                    Else
                        strfreq = freq.ToString("0.0", GCI)
                    End If
                End If
                g.RotateTransform(-90)
                dy = g.MeasureString(strfreq, f, 1000).Width
                g.DrawString(strfreq, f, Brushes.White, -dy - 2, x + dx)
                g.ResetTransform()
                ' -------------------------------------------------------- increment frequency
                If LogX Then
                    freq = freq * klog
                Else
                    freq = freq + klin
                End If
                ' -------------------------------------------------------- print decibels
                g.RotateTransform(-90)
                db = MindB + (MaxdB - MindB) * m_fht.BandsBuffer(b)
                g.DrawString(db.ToString("0.0", GCI), f, Brushes.White, -h + 5, x + dx)
                g.ResetTransform()
            Next
            '
            If SignalOverload Then
                g.DrawString("SIGNAL OVERLOAD", New Font("Arial", fs * 3), Brushes.White, pbox.Image.Width / 3.0F, h / 4.0F)
            End If
            '
            pbox.Invalidate()
            '
            ' -----------------------------------------------------------------------
            '  Calculate ponderations A, B, C, D and Z 
            '  Time = 30..60 uS (on 6000..12000 uS total)
            ' -----------------------------------------------------------------------
            CalcPonderations(m_fht.BandsBuffer, MindB, MaxdB)
            '
        End Sub

        Private Function RoundFreq(ByVal f As Single) As Single
            For Each rf As Single In RoundedFreq31
                If Math.Abs(rf - f) < f / 100 Then
                    f = rf
                End If
            Next
            Return f
        End Function

        Friend Function GetBandFreq(ByVal band As Int32) As Single
            Dim freq As Single = MinFreq
            Dim klog As Single = CSng((MaxFreq / MinFreq) ^ (1 / (NumBands - 1)))
            Dim klin As Single = CSng((MaxFreq - MinFreq) / (NumBands - 1))
            For b As Int32 = 0 To NumBands - 1
                If b = band Then
                    Return freq
                End If
                If LogX Then
                    freq = freq * klog
                Else
                    freq = freq + klin
                End If
            Next
        End Function

        Friend Function GetRoundedFreq(ByVal band As Int32) As Single
            'Return RoundFreq(GetBandFreq(band)) '02/01/2024
            'CalibrationPoints.Length()
            Band_CenterFrequency(NumBands, OldArrayFreq) '02/01/2024
            Return RoundFreq(OldArrayFreq(band)) '02/01/2024
        End Function

        Private Sub SetBandEqualizationsFromCalibrator()
            If m_fht.BandsEqualizations Is Nothing Then Return
            If m_fht.BandsEqualizations.Length = NumBands Then
                For i As Int32 = 0 To NumBands - 1
                    m_fht.BandsEqualizations(i) = GetMicCalibrationValue(NumBands, i)
                Next
            End If
        End Sub

    End Module
End Namespace
